package com.vau.biometric.vau_biometric

import android.content.Context
import android.content.Intent
import android.hardware.biometrics.BiometricManager.Authenticators.BIOMETRIC_STRONG
import androidx.annotation.NonNull;
import com.vau.biometric.vau_biometric.core.AuthProcess
import com.vau.biometric.vau_biometric.core.BiometricUtil
import com.vau.biometric.vau_biometric.model.AuthStatus
import com.vau.biometric.vau_biometric.view.AuthActivity

import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry.Registrar

/** VauBiometricPlugin */
class VauBiometricPlugin: FlutterPlugin, MethodCallHandler, ActivityAware {
  private lateinit var channel : MethodChannel

  override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
    channel = MethodChannel(flutterPluginBinding.flutterEngine.dartExecutor, "vau_biometric")
    channel.setMethodCallHandler(this)
    setChannel(channel)
  }

  companion object {
    @Volatile private var channel: MethodChannel ?= null
    private var context: Context ?= null

    @JvmStatic
    fun setChannel(methodChannel: MethodChannel) {
      channel = methodChannel
    }

    @JvmStatic
    fun getChannel() : MethodChannel {
      return channel!!
    }

    @JvmStatic
    fun setContext(activity: Context) {
      context = activity
    }
  }

  override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
    if (call.method == "beginBiometricLogin") {
      val type: Int? = call.argument<Int>("type")
      val title: String? = call.argument("title")
      val subTitle: String? = call.argument("subtitle")
      val negativeButtonText: String? = call.argument("negativeButtonText")
      when(BiometricUtil.getInstance().checkAuth(context!!)) {
        is AuthStatus.AuthSuccess -> {
          context!!.startActivity(Intent(context, AuthActivity::class.java)
                  .putExtra("type", type)
                  .putExtra("title", title)
                  .putExtra("subtitle", subTitle)
                  .putExtra("negativeButtonText", negativeButtonText))
          result.success("success")
        }
        is AuthStatus.AuthError -> {
          result.error("error","error", null)
        }
        is AuthStatus.AuthNoHardware -> {
          result.error("error", "noHardware", null)
        }
        is AuthStatus.AuthNoEnrolled -> {
          result.error("error", "noEnrolled", null)
        }
      }
    }
  }

  override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
    channel.setMethodCallHandler(null)
  }

  override fun onDetachedFromActivity() {

  }

  override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {

  }

  override fun onAttachedToActivity(binding: ActivityPluginBinding) {
    setContext(binding.activity)
  }

  override fun onDetachedFromActivityForConfigChanges() {

  }
}
